_G.CustomNameColor = _G.CustomNameColor or {}
local CNC = _G.CustomNameColor
CNC.ModID = "CustomNameColor"
CNC._path = (BeardLib and CNC:GetPath()) or ModPath
CNC._data_path = SavePath .. "CustomNameColor.txt"

CNC.default_palettes = {
	"de901f",
	"cdde1f",
	"6dde1f",
	"de301f",
	"de1f6d",
	"1f7ade",
	"241fde",
	"831fde",
	"1fdade",
	"1fde83",
	"de1fa3",
	"de1f43",
	"de5a1f",
	"ba1fde",
	"5a1fde",
	"1fde22",
	"1fde82",
	"1fdbde",
	"7cde1f",
	"dbde1f",
	"1fdede",
	"1f7fde",
	"1f1fde",
	"1fde7f",
	"1fde1f"
}
--create entries for all settings
CNC.default_settings = {
	enabled = true,
	compatibility_mode = false,
	own_r = 1,
	own_g = 0.25,
	own_b = 1, -- own color is equivalent to about FF40FF, or magenta
	bot_r = 0.2,
	bot_g = 0.8,
	bot_b = 1, --own color is basically cyan 
	palettes = table.deep_map_copy(CNC.default_palettes)
}
CNC._colorpickers = {} --only populated if ColorPicker is installed

CNC._own_preview = nil
CNC._bot_preview = nil
CNC._PREVIEW_SIZE = 80

--duplicate settings so that old saves after an update will still get any new settings entries
CNC.settings = CNC.settings or {}
for k,v in pairs(CNC.default_settings) do 
	if k == "palettes" then 
		for swatch_id,color_hex_string in ipairs(v) do 
			CNC.settings.palettes = CNC.settings.palettes or table.deep_map_copy(CNC.default_palettes)
			CNC.settings.palettes[swatch_id] = color_hex_string
		end
	else
		CNC.settings[k] = CNC.settings[k] or v
	end
end

CNC.chat_colors_3 = { --vanilla chat colors, but in join order. used for name colors
	Color("C2FC97"),--green
	Color("78B7CC"),--blue
	Color("B26859"),--red
	Color("CCA166"),--orange
	Color(0.2, 0.8, 1)
}

function CNC:IsEnabled()
	return CNC.settings.enabled
end

function CNC:IsCompatibilityMode()
	return self.settings.compatibility_mode
end

function CNC:GetBotColor()
	return Color(self.settings.bot_r,self.settings.bot_g,self.settings.bot_b)
end

function CNC:GetOwnColor()
	return Color(self.settings.own_r,self.settings.own_g,self.settings.own_b) --not sure about the alpha thing? maybe only force alpha 1 for other players?
end

function CNC:SetOwnColor(color)
	self.settings.own_r = color.r
	self.settings.own_g = color.g
	self.settings.own_b = color.b
end

function CNC:SetBotColor(color)
	self.settings.bot_r = color.r
	self.settings.bot_g = color.g
	self.settings.bot_b = color.b
end

function CNC:SetPeerColor(id,data)
	if id and data then
		local col = LuaNetworking:StringToColour(data)
		tweak_data.chat_colors[id] = col and col:with_alpha(1)
		self:RefreshInGameColors(id)
	end
end

function CNC:SetLocalColors()
	-- BigLobby expands chat_colors to the amount of players, bot color's bopped up to max player count + 1 so a hard 5 won't work.
	local id = managers.network:session():local_peer():id()
	tweak_data.chat_colors[id] = CustomNameColor:GetOwnColor()
	tweak_data.chat_colors[HUDManager.PLAYER_PANEL + 1] = CustomNameColor:GetBotColor()
end

function CNC:RefreshInGameColors(id)
	if managers.hud == nil or self:IsCompatibilityMode() then return end
	self:SetLocalColors();

	local briefing = managers.hud._hud_mission_briefing
	if briefing and self:IsEnabled() then
		-- Refresh the main player briefing color if an id was not provided
		local slot_id = id or managers.network:session():local_peer():id()
		local slot = briefing._ready_slot_panel:child("slot_" .. tostring(slot_id))
		
		local briefing_color = tweak_data.chat_colors[slot_id]
		if (slot and alive(slot)) and not _G.IGWSC then --for In-Game Waiting Status Color compatibility
--			slot:child("status"):set_color(briefing_color) --"READY" or "NOT READY"
			slot:child("name"):set_color(briefing_color)
			slot:child("criminal"):set_color(briefing_color)
			slot:child("infamy"):set_color(briefing_color)
		end
	end

	-- Since set_callsign has no side effects besides setting the colours, can just call it again and save ourselves some trouble.
	-- Also updates all of MUI's colours this way.
	for i,teammate_panel in pairs(managers.hud._teammate_panels or {}) do
		local peer_id = teammate_panel._peer_id or managers.network:session():local_peer():id()
		if teammate_panel._ai then peer_id = HUDManager.PLAYER_PANEL + 1 end

		teammate_panel:set_callsign(peer_id)
	end
end

function CNC:SyncColorToPeers()
	if LuaNetworking then 
		local col = LuaNetworking:ColourToString(self:GetOwnColor())
		LuaNetworking:SendToPeers(self.ModID,col)
	end
end

Hooks:Add("LocalizationManagerPostInit", "cnc_LocalizationManagerPostInit", function( loc )
	if not BeardLib then 
		loc:load_localization_file( CNC._path .. "localization/en.txt")
	end
end)

function CNC:Load()
	local file = io.open(self._data_path, "r")
	if (file) then
		for k, v in pairs(json.decode(file:read("*all"))) do
		
			if k == "palettes" then 
				for swatch_id,color_hex_string in ipairs(v) do 
					self.settings.palettes = self.settings.palettes or table.deep_map_copy(self.default_palettes)
					self.settings.palettes[swatch_id] = color_hex_string
				end
			else
				self.settings[k] = v
			end
			
		end
	else
		self:Save() --create data in case there's no mod save data
	end
end

function CNC:Save()
	local file = io.open(self._data_path,"w+")
	if file then
		file:write(json.encode(self.settings))
		file:close()
	end
end

Hooks:Add("NetworkReceivedData", "cnc_NetworkReceivedData", function(sender, message, data, ...)
	if message == CNC.ModID then
		if data ~= "" then
			CNC:SetPeerColor(sender,data)
		end
	end
	
end)

Hooks:Add("NetworkManagerOnPeerAdded", "cnc_NetworkManagerOnPeerAdded", function(peer, peer_id)
	CNC:SyncColorToPeers()
end)


function CNC:ShowOwnColorPreview(color)
	if not (managers.menu_component and managers.menu_component._ws and managers.menu_component._ws and alive(managers.menu_component._ws:panel())) then 
		return
	end
	
	local panel = managers.menu_component._ws:panel()
	
	color = color or self:GetOwnColor()
	
	local own_preview = self._own_preview
	if not alive(own_preview) then 
		own_preview = panel:panel({
			name = "customnamecolor_preview_own"
		})
		self._own_preview = own_preview
		
	--[[
		self._own_preview = panel:rect({
			name = "customnamecolor_preview_own",
			color = color or self:GetOwnColor(),
			x = 670,
			y = 180 - (self._PREVIEW_SIZE / 2),
			w = self._PREVIEW_SIZE,
			h = self._PREVIEW_SIZE,
			layer = tweak_data.gui.MENU_LAYER + 1
		})
		--]]
		
		local tabs_texture = "guis/textures/pd2/hud_tabs"
		local bg_rect = {
			84,
			0,
			44,
			32
		}
		local cs_rect = {
			84,
			34,
			19,
			19
		}
		local csbg_rect = {
			105,
			34,
			19,
			19
		}
		local bg_color = Color.white / 3
		local preview_text = own_preview:text({
			name = "preview_text",
			text = managers.network.account:username(),
			font = tweak_data.hud_players.name_font,
			font_size = tweak_data.hud_players.name_size,
			align = "right",
			color = color,
			x = 670,
			y = 180 - (self._PREVIEW_SIZE / 2),
			w = self._PREVIEW_SIZE,
			h = self._PREVIEW_SIZE,
			layer = tweak_data.gui.MENU_LAYER + 1
		})
		local tx,ty,tw,th = preview_text:text_rect()
		local callsign_size = th - 2
		local preview_text_bg = own_preview:bitmap({
			name = "preview_text_bg",
			layer = 0,
			texture = tabs_texture,
			texture_rect = bg_rect,
			color = bg_color,
			x = tx - 2,
			y = preview_text:y() - 1,
			w = tw + 4,
			h = th + 4
		})
		
		local callsign_x = tx + -callsign_size + -4
		local preview_callsign = own_preview:bitmap({
			name = "preview_callsign",
			layer = 1,
			blend_mode = "normal",
			texture = tabs_texture,
			texture_rect = cs_rect,
			color = color,
			x = callsign_x,
			y = preview_text:y() + 1,
			w = callsign_size,
			h = callsign_size
		})
		local preview_callsign_bg = own_preview:bitmap({
			name = "preview_callsign_bg",
			layer = 0,
			blend_mode = "normal",
			texture = tabs_texture,
			texture_rect = csbg_rect,
			color = bg_color,
			x = callsign_x,
			y = preview_text:y() + 1,
			w = callsign_size,
			h = callsign_size
		})
		
		return
	end
	
	own_preview:child("preview_callsign"):set_color(color)
	own_preview:child("preview_text"):set_color(color)
end

--CustomNameColor._bot_preview:set_position(650,300)
function CNC:ShowBotColorPreview(color)
	if not (managers.menu_component and managers.menu_component._ws and managers.menu_component._ws and alive(managers.menu_component._ws:panel())) then 
		return
	end
	
	local panel = managers.menu_component._ws:panel()
	
	color = color or self:GetBotColor()
	
	local bot_preview = self._bot_preview
	if not alive(bot_preview) then 
		bot_preview = panel:panel({
			name = "customnamecolor_preview_bot"
		})
		self._bot_preview = bot_preview
		
		local tabs_texture = "guis/textures/pd2/hud_tabs"
		local bg_rect = {
			84,
			0,
			44,
			32
		}
		local cs_rect = {
			84,
			34,
			19,
			19
		}
		local csbg_rect = {
			105,
			34,
			19,
			19
		}
		local bg_color = Color.white / 3
		
		local random_character_index = math.random(#tweak_data.criminals.characters)
		local char_data = tweak_data.criminals.characters[random_character_index] or {name="american"}
		local character_name = managers.localization:text("menu_" .. tostring(char_data.name))
		
		
		local preview_text = bot_preview:text({
			name = "preview_text",
			text = character_name,
			font = tweak_data.hud_players.name_font,
			font_size = tweak_data.hud_players.name_size,
			align = "right",
			color = color,
			x = 670,
			y = 270 - (self._PREVIEW_SIZE / 2),
			w = self._PREVIEW_SIZE,
			h = self._PREVIEW_SIZE,
			layer = tweak_data.gui.MENU_LAYER + 1
		})
		local tx,ty,tw,th = preview_text:text_rect()
		local callsign_size = th - 2
		local preview_text_bg = bot_preview:bitmap({
			name = "preview_text_bg",
			layer = 0,
			texture = tabs_texture,
			texture_rect = bg_rect,
			color = bg_color,
			x = tx - 2,
			y = preview_text:y() - 1,
			w = tw + 4,
			h = th + 4
		})
		
		local callsign_x = tx + -callsign_size + -4
		local preview_callsign = bot_preview:bitmap({
			name = "preview_callsign",
			layer = 1,
			blend_mode = "normal",
			texture = tabs_texture,
			texture_rect = cs_rect,
			color = color,
			x = callsign_x,
			y = preview_text:y() + 1,
			w = callsign_size,
			h = callsign_size
		})
		local preview_callsign_bg = bot_preview:bitmap({
			name = "preview_callsign_bg",
			layer = 0,
			blend_mode = "normal",
			texture = tabs_texture,
			texture_rect = csbg_rect,
			color = bg_color,
			x = callsign_x,
			y = preview_text:y() + 1,
			w = callsign_size,
			h = callsign_size
		})
		
		return
	end
	bot_preview:child("preview_callsign"):set_color(color)
	bot_preview:child("preview_text"):set_color(color)
end

function CNC:RemoveOwnPreview()
	if self._own_preview and alive(self._own_preview) then 
		self._own_preview:parent():remove(self._own_preview)
		self._own_preview = nil
	end
end

function CNC:RemoveBotPreview()
	if self._bot_preview and alive(self._bot_preview) then 
		self._bot_preview:parent():remove(self._bot_preview)
		self._bot_preview = nil
	end
end


function CNC:SetPalettes(palettes)
	for i,color in ipairs(palettes) do 
		self.settings.palettes[i] = ColorPicker.color_to_hex(color)
	end
end

function CNC:GetPalettes()
	local palettes = table.deep_map_copy(CNC.default_palettes)
	for i,color_string in ipairs(self.settings.palettes) do 
		palettes[i] = Color(color_string)
	end
	return palettes
end

function CNC:GetDefaultPalettes()
	local palettes = {}
	for i,color_string in ipairs(self.default_palettes) do 
		palettes[i] = Color(color_string)
	end
	return palettes
end

function CNC:CreateColorPickers()
	if _G.ColorPicker then 
		self._colorpickers.own = ColorPicker:new(
			"customnamecolor_own_colorpicker",
			{
				color = self:GetOwnColor(),
				palettes = self:GetPalettes(),
				default_palettes = self:GetDefaultPalettes(),
				done_callback = function(new_color,palettes,success)
					if success then 
						self:SetOwnColor(new_color)
						self:SetPalettes(palettes)
						self:Save()
					else
						--resets own color preview to current own color
						self:ShowOwnColorPreview()
					end
				end,
				changed_callback = function(new_color)
					self:ShowOwnColorPreview(new_color)
				end
			},
			function(obj)
				self._colorpickers.own = self._colorpickers.own or obj
			end
		)
		
		self._colorpickers.bot = ColorPicker:new(
			"customnamecolor_bot_colorpicker",
			{
				color = self:GetBotColor(),
				palettes = self:GetPalettes(),
				default_palettes = self:GetDefaultPalettes(),
				done_callback = function(new_color,palettes,success)
					if success then 
						self:SetBotColor(new_color)
						self:SetPalettes(palettes)
						self:Save()
					else
						--resets bot color preview to current bot color
						self:ShowBotColorPreview()
					end
				end,
				changed_callback = function(new_color)
					self:ShowBotColorPreview(new_color)
				end
			},
			function(obj)
				self._colorpickers.bot = self._colorpickers.bot or obj
			end
		)
		
	end
end

function CNC:callback_show_colorpicker_own()
	if self._colorpickers.own then 
		self._colorpickers.own:Show({palettes = self:GetPalettes(),color=self:GetOwnColor()})
		self:ShowOwnColorPreview()
	else
		self:callback_show_colorpicker_prompt()
	end
end

function CNC:callback_show_colorpicker_bot()
	if self._colorpickers.bot then 
		self._colorpickers.bot:Show({palettes = self:GetPalettes(),color=self:GetBotColor()})
		self:ShowBotColorPreview()
	else
		self:callback_show_colorpicker_prompt()
	end
end

function CNC:callback_show_colorpicker_prompt()
	QuickMenu:new(managers.localization:text("menu_cnc_missing_colorpicker_title"),string.gsub(managers.localization:text("menu_cnc_missing_colorpicker_desc"),"$URL","https://modworkshop.net/mod/29641"),{
		{
			text = managers.localization:text("menu_cnc_ok"),
			is_cancel_button = true
		}
	},true)
end


Hooks:Add( "MenuManagerInitialize", "MenuManagerInitialize_cnc", function(menu_manager)
	CNC:CreateColorPickers()
	
	
	MenuCallbackHandler.callback_cnc_enabled = function(self,item)
		local state = item:value() == "on"
		CNC.settings.enabled = state
		if not state then 
			CNC:RefreshInGameColors()
		else
			CNC:SyncColorToPeers()
		end
		CNC:Save()
	end

	MenuCallbackHandler.callback_cnc_compatibility_mode = function(self,item)
		CNC.settings.compatibility_mode = item:value() == "on"
		CNC:Save()
	end

	MenuCallbackHandler.callback_cnc_own_r_slider = function(self,item)
		CNC.settings.own_r = item:value()
		CNC:ShowOwnColorPreview()
		CNC:Save()
	end
	MenuCallbackHandler.callback_cnc_own_g_slider = function(self,item)
		CNC.settings.own_g = item:value()
		CNC:ShowOwnColorPreview()
		CNC:Save()
	end
	MenuCallbackHandler.callback_cnc_own_b_slider = function(self,item)
		CNC.settings.own_b = item:value()
		CNC:ShowOwnColorPreview()
		CNC:Save()
	end
	MenuCallbackHandler.callback_cnc_show_own_preview = function(self,item)
		CNC:ShowOwnColorPreview()
	end
	
	MenuCallbackHandler.callback_cnc_bot_r_slider = function(self,item)
		CNC.settings.bot_r = item:value()
		CNC:ShowBotColorPreview()
		CNC:Save()
	end
	MenuCallbackHandler.callback_cnc_bot_g_slider = function(self,item)
		CNC.settings.bot_g = item:value()
		CNC:ShowBotColorPreview()
		CNC:Save()
	end
	MenuCallbackHandler.callback_cnc_bot_b_slider = function(self,item)
		CNC.settings.bot_b = item:value()
		CNC:ShowBotColorPreview()
		CNC:Save()
	end
	
	MenuCallbackHandler.callback_cnc_show_bot_preview = function(self,item)
		CNC:ShowBotColorPreview()
	end
	
	MenuCallbackHandler.callback_cnc_open_own_colorpicker = function(self,item)
		CNC:callback_show_colorpicker_own()
	end
	
	MenuCallbackHandler.callback_cnc_open_bot_colorpicker = function(self,item)
		CNC:callback_show_colorpicker_bot()
	end
	
	MenuCallbackHandler.callback_cnc_focus_changed = function(self,focused)
		if focused then 
			CNC:ShowOwnColorPreview()
			CNC:ShowBotColorPreview()
		else
			CNC:RemoveOwnPreview()
			CNC:RemoveBotPreview()
		end
	end
	
	MenuCallbackHandler.callback_cnc_close = function(this)
--		CNC:RemoveOwnPreview()
--		CNC:RemoveBotPreview()
		if CNC:IsEnabled() then 
			CNC:RefreshInGameColors()
		end
	end

	CNC:Load()
	MenuHelper:LoadFromJsonFile(CNC._path .. "options.txt", CNC, CNC.settings)	
	
end)